@@ -4,17 +4,16 @@ import operator
 import colorsys
 from enum import Enum
 
+
 class AgentShape(Enum):
     CIRCLE = 1
     ARROW = 2
     PERSON = 3
     HOUSE = 4
 
+
 class Agent:
     def __init__(self):
-        # Associated simulation area.
-        self.__model = None
-
         # Destroyed agents are not drawn and are removed from their area.
         self.__destroyed = False
 
@@ -27,18 +26,21 @@ class Agent:
         hue = random.random()
         saturation = random.uniform(0.8, 1.0)
         lightness = random.uniform(0.25, 1.0)
-        r,g,b = colorsys.hls_to_rgb(hue, lightness, saturation)
+        r, g, b = colorsys.hls_to_rgb(hue, lightness, saturation)
         self.__color = (int(r*255), int(g*255), int(b*255))
 
         self.x = 0
         self.y = 0
         self.size = 8
-        self.direction = random.randint(0, 359)
+        self.__direction = random.randint(0, 359)
         self.speed = 1
         self.__current_tile = None
         self.selected = False
         self.shape = AgentShape.ARROW
 
+        # Associated simulation area.
+        get_quickstart_model().add_agent(self, setup=False)
+
     # Should be overwritten by a subclass
     def setup(self, model):
         pass
@@ -47,7 +49,7 @@ class Agent:
     def update_current_tile(self):
         new_tile = self.current_tile()
         if not self.__current_tile:
-            self.__current_tile = self.current_tile()
+            self.__current_tile = new_tile
             self.__current_tile.add_agent(self)
         elif not (self.__current_tile is new_tile):
             self.__current_tile.remove_agent(self)
@@ -70,36 +72,52 @@ class Agent:
     # If the agent is outside the simulation area,
     # return it to the closest point inside
     def __stay_inside(self):
-        self.x = min(max(self.x,0),self.__model.width)
-        self.y = min(max(self.y,0),self.__model.height)
+        self.x = min(max(self.x, 0), self.__model.width)
+        self.y = min(max(self.y, 0), self.__model.height)
 
-    def align(self):
+    def center_in_tile(self):
+        """
+        Move the agent to the center of the tile it is standing on
+        """
         w = self.__model.width
         h = self.__model.height
         tx = self.__model.x_tiles
         ty = self.__model.y_tiles
         self.x = math.floor(self.x * tx / w) * w / tx + (w / tx) / 2
         self.y = math.floor(self.y * ty / h) * h / ty + (h / ty) / 2
+        self.__post_move()
 
     def jump_to(self, x, y):
+        """
+        Move the agent to a specified point
+        """
         self.x = x
         self.y = y
         self.__post_move()
 
     def jump_to_tile(self, t):
+        """
+        Move the agent to the center of a specified tile
+        """
         w = self.__model.width
         h = self.__model.height
         x_tiles = self.__model.x_tiles
         y_tiles = self.__model.y_tiles
         self.x = t.x * w / x_tiles
         self.y = t.y * h / y_tiles
-        self.align()
+        self.center_in_tile()
         self.__post_move()
 
     def set_model(self, model):
+        """
+        Set the model of the agent
+        """
         self.__model = model
 
     def direction_to(self, other_x, other_y):
+        """
+        Calculate the direction in degrees from the agent to a given point
+        """
         direction = 0
         dist = self.distance_to(other_x, other_y)
         if dist > 0:
@@ -109,23 +127,58 @@ class Agent:
         return direction
 
     def point_towards(self, other_x, other_y):
+        """
+        Make the agent orient itself towards a given point
+        """
         dist = self.distance_to(other_x, other_y)
         if dist > 0:
             self.direction = math.degrees(math.acos((other_x - self.x) / dist))
             if (self.y - other_y) > 0:
                 self.direction = 360 - self.direction
 
-    def forward(self):
-        self.x += math.cos(math.radians(self.direction)) * self.speed
-        self.y += math.sin(math.radians(self.direction)) * self.speed
+    def forward(self, distance=None):
+        """
+        Moves the agent forward in the direction it is currently facing.
+        """
+        if distance is None:
+            distance = self.speed
+        self.x += math.cos(math.radians(self.direction)) * distance
+        self.y += math.sin(math.radians(self.direction)) * distance
         self.__post_move()
 
+    def backward(self, distance=None):
+        """
+        Moves the agent in the opposite direction of its current orientation
+        """
+        if distance is None:
+            distance = self.speed
+        self.x -= math.cos(math.radians(self.direction)) * distance
+        self.y -= math.sin(math.radians(self.direction)) * distance
+        self.__post_move()
+
+    def left(self, degrees):
+        """
+        Make the agent turn left (counter clockwise) the given number of degrees
+        """
+        self.direction += degrees
+
+    def right(self, degrees):
+        """
+        Make the agent turn right (clockwise) the given number of degrees
+        """
+        self.direction -= degrees
+
     def distance_to(self, other_x, other_y):
+        """
+        Returns the distance between the agent and another point.
+        """
         return ((self.x - other_x) ** 2 + (self.y - other_y) ** 2) ** 0.5
 
-    # Returns a list of nearby agents.
-    # May take a type as argument and only return agents of that type.
     def agents_nearby(self, distance, agent_type=None):
+        """
+        Returns a list of nearby agents.
+        May take a type as argument and only return agents of that type.
+        """
         nearby = set()
         for a in self.__model.agents:
             if self.distance_to(a.x, a.y) <= distance and not (a is self):
@@ -134,33 +187,40 @@ class Agent:
         return nearby
 
     def current_tile(self):
-        x = (
-            math.floor(self.__model.x_tiles * self.x / self.__model.width)
-            % self.__model.x_tiles
-        )
-        y = (
-            math.floor(self.__model.y_tiles * self.y / self.__model.height)
-            % self.__model.y_tiles
-        )
-        i = y * self.__model.x_tiles + x
-        return self.__model.tiles[i]
+        """
+        Returns the tile that the agent is currently standing on.
+        """
+        x = math.floor((self.__model.x_tiles * self.x) / self.__model.width)
+        y = math.floor((self.__model.y_tiles * self.y) / self.__model.height)
+        return self.__model.tile(x, y)
 
-    # Returns the surrounding tiles as a 3x3 grid. Includes the current tile.
     def neighbor_tiles(self):
+        """
+        Returns the surrounding tiles as a 3x3 grid. Includes the current tile.
+        """
         return self.nearby_tiles(-1, -1, 1, 1)
 
     def nearby_tiles(self, x1, y1, x2, y2):
-        t = self.__current_tile
+        """
+        Returns a rectangle of tiles relative to the agent's current position.
+        """
+        tile = self.__current_tile
         tiles = []
-        for y in range(y1, y2 + 1):
-            row = self.__model.x_tiles * ((t.y + y) % self.__model.y_tiles)
-            tiles += self.__model.tiles[(row + t.x + x1):(row + t.x + x2 + 1)]
+        for x in range(x1, x2 + 1):
+            for y in range(y1, y2 + 1):
+                tiles.append(self.__model.tile((tile.x + x), (tile.y + y)))
         return tiles
 
     def is_destroyed(self):
+        """
+        Returns True or false whether or not the agent is destroyed
+        """
         return self.__destroyed
 
     def destroy(self):
+        """
+        Marks the agent for destruction, removing it from the set of agents in the model.
+        """
         if not self.__destroyed:
             self.__destroyed = True
 
@@ -173,6 +233,14 @@ class Agent:
         r, g, b = color
         self.__color = [r, g, b]
 
+    @property
+    def direction(self):
+        return self.__direction % 360
+
+    @direction.setter
+    def direction(self, direction):
+        self.__direction = direction % 360
+
 
 class Tile:
     def __init__(self, x, y, model):
@@ -210,7 +278,7 @@ class ToggleSpec(Spec):
 
 
 class SliderSpec(Spec):
-    def __init__(self, variable, minval, maxval, initial):
+    def __init__(self, variable, initial, minval, maxval):
         self.variable = variable
         self.minval = minval
         self.maxval = maxval
@@ -223,9 +291,11 @@ class CheckboxSpec(Spec):
 
 
 class LineChartSpec(Spec):
-    def __init__(self, variable, color):
-        self.variable = variable
-        self.color = color
+    def __init__(self, variables, colors, min_y, max_y):
+        self.variables = variables
+        self.colors = colors
+        self.min_y = min_y
+        self.max_y = max_y
 
 
 class MonitorSpec(Spec):
@@ -251,9 +321,25 @@ class HistogramSpec(Spec):
         ]
         self.color = color
 
+class EllipseStruct():
+    def __init__(self,x,y,w,h,color):
+        self.x = x
+        self.y = y
+        self.w = w
+        self.h = h
+        self.color = color
+
+class RectStruct():
+    def __init__(self,x,y,w,h,color):
+        self.x = x
+        self.y = y
+        self.w = w
+        self.h = h
+        self.color = color
 
 class Model:
-    def __init__(self, title, x_tiles=50, y_tiles=50, tile_size=8, cell_data_file=None):
+    def __init__(self, title, x_tiles=50, y_tiles=50, tile_size=8,
+                 cell_data_file=None):
         # Title of model, shown in window title
         self.title = title
 
@@ -262,49 +348,34 @@ class Model:
             self.x_tiles = x_tiles
             self.y_tiles = y_tiles
 
-            # Pixel sizes
-            self.tile_size = tile_size
-            self.width = x_tiles * tile_size
-            self.height = y_tiles * tile_size
-
-            # Initial tileset (empty).
-            self.tiles = [Tile(x, y, self)
-                          for y in range(y_tiles)
-                          for x in range(x_tiles)]
         else:
             cell_data = open(cell_data_file, "r")
             cell_data.readline()
             cell_data.readline()
-            self.header_info = ( cell_data.readline()[:-1]).split('\t')
-            x_tiles = 0
-            y_tiles = 0
+            header_line = cell_data.readline()[:-1]
+            self.header_info = header_line.split('\t')
 
             self.load_data = []
             for line in cell_data:
                 cell = line[:-1].split('\t')
                 x = int(cell[0])
                 y = int(cell[1])
-                x_tiles = max(x+1, x_tiles)
-                y_tiles = max(y+1, y_tiles)
+                self.x_tiles = max(x+1, x_tiles)
+                self.y_tiles = max(y+1, y_tiles)
                 self.load_data.append(cell)
 
-            # Pixel sizes
-            self.tile_size = tile_size
-            self.width = x_tiles * tile_size
-            self.height = y_tiles * tile_size
-            self.x_tiles = x_tiles
-            self.y_tiles = y_tiles
-
-            # Internal set of agents.
-            self.__agents = set()
-
-            # Initial tileset (empty).
-            self.tiles = [Tile(x, y, self)
-                          for y in range(y_tiles)
-                          for x in range(x_tiles)]
-
             cell_data.close()
 
+        # Initial tileset (empty).
+        self.tiles = [Tile(x, y, self)
+                      for y in range(y_tiles)
+                      for x in range(x_tiles)]
+
+        # Pixel sizes
+        self.tile_size = tile_size
+        self.width = x_tiles * tile_size
+        self.height = y_tiles * tile_size
+
         self.__agents = []
         self.variables = {}
         self.plot_specs = []
@@ -315,19 +386,26 @@ class Model:
         self._paused = False
         self._wrapping = True
         self._close_func = None
+        self._shapes = []
 
-    def add_agent(self, agent):
+    def add_agent(self, agent, setup=True):
         agent.set_model(self)
         agent.x = random.randint(0, self.width)
         agent.y = random.randint(0, self.height)
         agent.update_current_tile()
         self.__agents.append(agent)
-        agent.setup(self)
+        if setup:
+            agent.setup(self)
 
     def add_agents(self, agents):
         for a in agents:
             self.add_agent(a)
 
+    def tile(self, x, y):
+        row = y % self.y_tiles
+        column = x % self.x_tiles
+        return self.tiles[row * self.x_tiles + column]
+
     # Based on
     # kite.com
     # /python
@@ -346,29 +424,35 @@ class Model:
         for a in self.__agents:
             a.destroy()
         self.__agents = []
+        self._shapes = []
         for x in range(self.x_tiles):
             for y in range(self.y_tiles):
                 i = y * self.x_tiles + x
                 self.tiles[i].color = (0, 0, 0)
                 self.tiles[i].info = {}
+        self.clear_plots()
         self.unpause()
 
     def reload(self):
         for tile_data in self.load_data:
             x = int(tile_data[0])
             y = int(tile_data[1])
-            for i in range(2,len(tile_data)):
+            for i in range(2, len(tile_data)):
                 variable = self.header_info[i]
-                self.tiles[y*self.x_tiles+x].info[variable] = float(tile_data[i])
+                tile = self.tiles[y*self.x_tiles+x]
+                tile.info[variable] = float(tile_data[i])
 
     def update_plots(self):
         for plot in self.plots:
             if type(plot.spec) is LineChartSpec:
-                plot.add_data(self.variables[plot.spec.variable])
+                dataset = []
+                for d in plot.spec.variables:
+                    dataset.append(getattr(self,d))
+                plot.add_data(dataset)
             elif type(plot.spec) is BarChartSpec:
                 dataset = []
                 for d in plot.spec.variables:
-                    dataset.append(self.variables[d])
+                    dataset.append(getattr(self,d))
                 plot.update_data(dataset)
             elif type(plot.spec) is HistogramSpec:
                 dataset = []
@@ -418,16 +502,26 @@ class Model:
     def add_toggle_button(self, label, func):
         self.current_row.append(ToggleSpec(label, func))
 
-    def add_slider(self, variable, minval, maxval, initial):
+    def add_slider(self, variable, initial, minval=0, maxval=100):
+        if len(self.current_row) > 0:
+            self.add_controller_row()
+        setattr(self, variable, initial)
         self.variables[variable] = initial
-        self.current_row.append(SliderSpec(variable, minval, maxval, initial))
+        self.current_row.append(SliderSpec(variable, initial, minval, maxval))
 
     def add_checkbox(self, variable):
-        self.variables[variable] = False
+        if len(self.current_row) > 0:
+            self.add_controller_row()
+        setattr(self, variable, False)
         self.current_row.append(CheckboxSpec(variable))
 
-    def line_chart(self, variable, color):
-        self.plot_specs.append(LineChartSpec(variable, color))
+    def monitor(self, variable):
+        if len(self.current_row) > 0:
+            self.add_controller_row()
+        self.current_row.append(MonitorSpec(variable))
+
+    def line_chart(self, variables, colors, min_y=None, max_y=None):
+        self.plot_specs.append(LineChartSpec(variables, colors, min_y, max_y))
 
     def bar_chart(self, variables, color):
         self.plot_specs.append(BarChartSpec(variables, color))
@@ -437,8 +531,21 @@ class Model:
             HistogramSpec(variable, minimum, maximum, bins, color)
         )
 
-    def monitor(self, variable):
-        self.current_row.append(MonitorSpec(variable))
+    def add_ellipse(self,x,y,w,h,color):
+        new_shape = EllipseStruct(x,y,w,h,color)
+        self._shapes.append(new_shape)
+        return new_shape
+
+    def add_rect(self,x,y,w,h):
+        new_shape = RectStruct(x,y,w,h,color)
+        self._shapes.append(new_shape)
+        return new_shape
+
+    def get_shapes(self):
+        return iter(self._shapes)
+
+    def clear_shapes(self):
+        self._shapes.clear()
 
     def is_paused(self):
         return self._paused
@@ -453,6 +560,7 @@ class Model:
         self._close_func = func
 
     def close(self):
+        self.pause()
         if self._close_func:
             self._close_func(self)
 
@@ -504,3 +612,62 @@ class SimpleModel(Model):
 
         self.add_button("Setup", setup_wrapper)
         self.add_toggle_button("Go", step_wrapper)
+
+
+def get_quickstart_model():
+    global quickstart_model
+    if 'quickstart_model' not in globals():
+        quickstart_model = Model("AgentsPy model", 50, 50)
+    return quickstart_model
+
+def contains_agent_type(agents, agent_type):
+    """
+    Returns a boolean indicating whether an agent of agent_type is in agents.
+    """
+    for a in agents:
+        if type(a) == agent_type:
+            return True
+    return False
+
+def only_agents_type(agents, agent_type):
+    """
+    Returns agents where all agents of type agent_type are removed.
+    """
+    agents_t = []
+    for a in agents:
+        if type(a) == agent_type:
+            agents_t.add(a)
+    return agents_t
+
+def remove_agents_type(agents, agent_type):
+    """
+    Returns agents where all agents not of type agent_type are removed.
+    """
+    agents_t = []
+    for a in agents:
+        if not (type(a) == agent_type):
+            agents_t.add(a)
+    return agents_t
+
+# kite.com/python/answers/how-to-sort-a-list-of-objects-by-attribute-in-python
+def agents_ordered(agents, variable, increasing=True):
+    """
+    Returns the agents list, sorted by variable in either increasing
+    or decreasing order. Prints an error if not all agents in the list
+    have the attribute.
+    """
+    try:
+        sorted_agents = sorted(list(agents), key=operator.attrgetter(variable))
+        if not increasing:
+            sorted_agents.reverse()
+        return iter(sorted_agents)
+    except:
+        print("Failed to sort agents. Do all agents have the attribute "+variable+" ?")
+        return agents
+
+def destroy_agents(agents):
+    """
+    Destroys all agents in agents.
+    """
+    for a in agents:
+        a.destroy()
