@@ -14,6 +14,7 @@ from PyQt5.QtCore import QPointF
 from PyQt5.QtGui import QPainter, QPainterPath, QColor, QPolygonF
 
 from agents.model import (
+    get_quickstart_model,
     AgentShape,
     ButtonSpec,
     ToggleSpec,
@@ -23,8 +24,11 @@ from agents.model import (
     BarChartSpec,
     HistogramSpec,
     MonitorSpec,
+    EllipseStruct,
+    RectStruct,
 )
 
+
 class SimulationArea(QtWidgets.QWidget):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -38,23 +42,31 @@ class SimulationArea(QtWidgets.QWidget):
     def model(self, model):
         self.__model = model
         self.setFixedSize(model.width, model.height)
+        self.enable_rendering = True
 
     def paintEvent(self, e):
         painter = QtGui.QPainter(self)
         painter.setPen(QtCore.Qt.NoPen)
         painter.setRenderHint(QtGui.QPainter.Antialiasing)
 
-        # Default to a white background
-        white = QtGui.QColor("white")
+        # Default to a black background
+        white = QtGui.QColor("black")
         painter.setBrush(white)
         painter.drawRect(0, 0,
-                         painter.device().width(),
-                         painter.device().height())
+                         painter.device().width(), painter.device().height())
 
         if self.model:
             # Draw tiles
             for tile in self.model.tiles:
                 self.paintTile(painter, tile)
+            # Draw shapes
+            for shape in self.model.get_shapes():
+                c = shape.color
+                painter.setBrush(QtGui.QColor(c[0], c[1], c[2]))
+                if type(shape) is EllipseStruct:
+                    painter.drawEllipse(shape.x,shape.y,shape.w,shape.h)
+                elif type(shape) is RectStruct:
+                    painter.drawRect(shape.x,shape.y,shape.w,shape.h)
             # Draw agents
             select = None
             for agent in self.model.agents:
@@ -95,7 +107,7 @@ class SimulationArea(QtWidgets.QWidget):
                 QPointF(x + math.cos(d + 2.3) * s, y + math.sin(d + 2.3) * s),
                 QPointF(
                     x + math.cos(d + math.pi) * s / 2,
-                    y + math.sin(d + math.pi) * s / 2
+                    y + math.sin(d + math.pi) * s / 2,
                 ),
                 QPointF(x + math.cos(d - 2.3) * s, y + math.sin(d - 2.3) * s),
             ]
@@ -105,40 +117,40 @@ class SimulationArea(QtWidgets.QWidget):
             y = agent.y - agent.size / 2
             size = agent.size
             point_list = [
-                QPointF(x + 0.4*size, y + 0.4*size),
-                QPointF(x + 0.2*size, y + 0.5*size),
-                QPointF(x + 0.2*size, y + 0.6*size),
-                QPointF(x + 0.4*size, y + 0.5*size),
-                QPointF(x + 0.4*size, y + 0.8*size),
-                QPointF(x + 0.2*size, y + size),
-                QPointF(x + 0.3*size, y + size),
-                QPointF(x + 0.5*size, y + 0.85*size),
-                QPointF(x + 0.7*size, y + size),
-                QPointF(x + 0.8*size, y + size),
-                QPointF(x + 0.6*size, y + 0.8*size),
-                QPointF(x + 0.6*size, y + 0.5*size),
-                QPointF(x + 0.8*size, y + 0.6*size),
-                QPointF(x + 0.8*size, y + 0.5*size),
-                QPointF(x + 0.6*size, y + 0.4*size),
+                QPointF(x + 0.4 * size, y + 0.4 * size),
+                QPointF(x + 0.2 * size, y + 0.5 * size),
+                QPointF(x + 0.2 * size, y + 0.6 * size),
+                QPointF(x + 0.4 * size, y + 0.5 * size),
+                QPointF(x + 0.4 * size, y + 0.8 * size),
+                QPointF(x + 0.2 * size, y + size),
+                QPointF(x + 0.3 * size, y + size),
+                QPointF(x + 0.5 * size, y + 0.85 * size),
+                QPointF(x + 0.7 * size, y + size),
+                QPointF(x + 0.8 * size, y + size),
+                QPointF(x + 0.6 * size, y + 0.8 * size),
+                QPointF(x + 0.6 * size, y + 0.5 * size),
+                QPointF(x + 0.8 * size, y + 0.6 * size),
+                QPointF(x + 0.8 * size, y + 0.5 * size),
+                QPointF(x + 0.6 * size, y + 0.4 * size),
             ]
             painter.drawPolygon(QPolygonF(point_list))
-            painter.drawEllipse(x + 0.3*size, y, 0.4*size, 0.4*size)
+            painter.drawEllipse(x + 0.3 * size, y, 0.4 * size, 0.4 * size)
         elif agent.shape == AgentShape.HOUSE:
             x = agent.x - agent.size / 2
             y = agent.y - agent.size / 2
             size = agent.size
             point_list = [
-                QPointF(x + 0.5*size, y),
-                QPointF(x, y + 0.5*size),
-                QPointF(x + 0.2*size, y + 0.5*size),
-                QPointF(x + 0.2*size, y + size),
-                QPointF(x + 0.40*size, y + size),
-                QPointF(x + 0.40*size, y + 0.7*size),
-                QPointF(x + 0.60*size, y + 0.7*size),
-                QPointF(x + 0.60*size, y + size),
-                QPointF(x + 0.8*size, y + size),
-                QPointF(x + 0.8*size, y + 0.5*size),
-                QPointF(x + size, y + 0.5*size)
+                QPointF(x + 0.5 * size, y),
+                QPointF(x, y + 0.5 * size),
+                QPointF(x + 0.2 * size, y + 0.5 * size),
+                QPointF(x + 0.2 * size, y + size),
+                QPointF(x + 0.40 * size, y + size),
+                QPointF(x + 0.40 * size, y + 0.7 * size),
+                QPointF(x + 0.60 * size, y + 0.7 * size),
+                QPointF(x + 0.60 * size, y + size),
+                QPointF(x + 0.8 * size, y + size),
+                QPointF(x + 0.8 * size, y + 0.5 * size),
+                QPointF(x + size, y + 0.5 * size),
             ]
             painter.drawPolygon(QPolygonF(point_list))
 
@@ -151,32 +163,46 @@ class SimulationArea(QtWidgets.QWidget):
         painter.drawRect(x, y, self.model.tile_size, self.model.tile_size)
 
     def mousePressEvent(self, e):
-        self.model.mouse_click(e.localPos().x(), e.localPos().y())
+        x = e.localPos().x()
+        y = e.localPos().y()
+        self.model.mouse_click(x, y)
 
+    def contextMenuEvent(self, event):
+        # Create menu
+        menu = QtWidgets.QMenu(self)
+        if self.enable_rendering:
+            pause_action = menu.addAction("Pause rendering")
+        else:
+            pause_action = menu.addAction("Start rendering")
+        # Open menu
+        action = menu.exec_(self.mapToGlobal(event.pos()))
+        # Handle user choice
+        if action == pause_action:
+            self.enable_rendering = not self.enable_rendering
 
 class QtGraph(QChartView):
     def __init__(self, spec):
         super().__init__(None)
         self.spec = spec
         self.chart = QChart()
-        self.chart.createDefaultAxes()
-        self.chart.setTitle(self.spec.variable)
+        self.chart.setTitle(str(self.spec.variables))
         self.chart.legend().hide()
-        self.series = QLineSeries()
-        self.series.setColor(QColor(spec.color[0],
-                                    spec.color[1],
-                                    spec.color[2]))
+
+        for color in self.spec.colors:
+            series = QLineSeries()
+            series.setColor(QColor(color[0], color[1], color[2]))
+            self.chart.addSeries(series)
+
         self.setMinimumWidth(400)
         self.setMinimumHeight(230)
 
-        self.axis_x = QValueAxis()
-        self.axis_y = QValueAxis()
-        self.chart.addSeries(self.series)
-        self.chart.setAxisX(self.axis_x, self.series)
-        self.chart.setAxisY(self.axis_y, self.series)
-
         self.setChart(self.chart)
         self.setRenderHint(QPainter.Antialiasing)
+        self.chart.createDefaultAxes()
+        self.autoscale_y_axis = True
+        if self.spec.min_y and self.spec.max_y:
+            self.autoscale_y_axis = False
+            self.chart.axes()[1].setRange(self.spec.min_y,self.spec.max_y)
 
         self._updates_per_second = 60
         self._data = []
@@ -184,7 +210,8 @@ class QtGraph(QChartView):
         self._max = 0
 
     def clear(self):
-        self.series.clear()
+        for chart in self.chart.series():
+            chart.clear()
         self._data = []
 
     def add_data(self, data):
@@ -192,19 +219,23 @@ class QtGraph(QChartView):
 
     def redraw(self):
         if len(self._data) > 0:
-            datapoint = sum(self._data) / len(self._data)
-            self.series.append(QPointF(self.series.count() /
-                                       self._updates_per_second, datapoint))
-            self.axis_x.setRange(0, (self.series.count() - 1)
-                                 / self._updates_per_second)
-            self._min = min(self._min, datapoint)
-            self._max = max(self._max, datapoint)
-            diff = self._max - self._min
+            for i in range(len(self.spec.variables)):
+                data = [datapoint[i] for datapoint in self._data]
+                datapoint = sum(data) / len(data)
+                self.chart.series()[i].append(QPointF(self.chart.series()[i].count() /
+                                                    self._updates_per_second,
+                                                    datapoint))
+                self._min = min(self._min, datapoint)
+                self._max = max(self._max, datapoint)
+        self.chart.axes()[0].setRange(0, (self.chart.series()[0].count() - 1) /
+                                      self._updates_per_second)
+        diff = self._max - self._min
+        if self.autoscale_y_axis:
             if diff > 0:
-                self.axis_y.setRange(self._min, self._max)
+                self.chart.axes()[1].setRange(self._min,self._max)
             else:
-                self.axis_y.setRange(self._min-0.5, self._max+0.5)
-            self._data = []
+                self.chart.axes()[1].setRange(self._min-0.5,self._max+0.5)
+        self._data = []
 
 
 class QtBarChart(QChartView):
@@ -216,9 +247,9 @@ class QtBarChart(QChartView):
         self.chart.legend().hide()
         self.mainset = QBarSet("")
         self.mainset.append([0 for i in range(len(spec.variables))])
-        self.mainset.setColor(QColor(spec.color[0],
-                                     spec.color[1],
-                                     spec.color[2]))
+        self.mainset.setColor(
+            QColor(spec.color[0], spec.color[1], spec.color[2])
+        )
         self.series = QBarSeries()
         self.series.append(self.mainset)
 
@@ -235,7 +266,7 @@ class QtBarChart(QChartView):
         self.setChart(self.chart)
         self.setRenderHint(QPainter.Antialiasing)
 
-        self._updates_per_second = 60
+        self._updates_per_second = 10
         self._dataset = []
 
     def clear(self):
@@ -264,9 +295,9 @@ class QtHistogram(QChartView):
 
         self.mainset = QBarSet("")
         self.mainset.append([0 for i in range(len(spec.bins))])
-        self.mainset.setColor(QColor(spec.color[0],
-                                     spec.color[1],
-                                     spec.color[2]))
+        self.mainset.setColor(
+            QColor(spec.color[0], spec.color[1], spec.color[2])
+        )
         self.series = QBarSeries()
         self.series.append(self.mainset)
 
@@ -283,7 +314,7 @@ class QtHistogram(QChartView):
         self.setChart(self.chart)
         self.setRenderHint(QPainter.Antialiasing)
 
-        self._updates_per_second = 60
+        self._updates_per_second = 10
         self._dataset = []
 
     def clear(self):
@@ -346,6 +377,7 @@ class Checkbox(QtWidgets.QCheckBox):
         super().__init__(variable)
         self.setText(variable)
 
+
 class MainWindow(QtWidgets.QMainWindow):
     def __init__(self, model):
         self.model = model
@@ -355,6 +387,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.model.close()
         super().closeEvent(event)
 
+
 class Application:
     def __init__(self, model):
         self.model = model
@@ -407,18 +440,21 @@ class Application:
 
         # Start timers
         self.logic_timer.start(1000 / 60)
-        self.graphics_timer.start(1000 / 60)
+        self.graphics_timer.start(1000 / 10)
 
     def update_logic(self):
         if not self.model.is_paused():
             for controller in self.controllers:
-                if isinstance(controller, ToggleButton) and controller.isChecked():
+                if (
+                    isinstance(controller, ToggleButton)
+                    and controller.isChecked()
+                ):
                     controller.func(controller.model)
                 elif isinstance(controller, Monitor):
                     controller.update_label()
 
     def update_graphics(self):
-        if not self.toggle_render_btn.isChecked():
+        if self.simulation_area.enable_rendering:
             self.simulation_area.update()
         for p in self.model.plots:
             p.redraw()
@@ -444,7 +480,7 @@ class Application:
 
         def update_variable(v):
             value = v / slider.sliderBar.factor
-            self.model[slider_spec.variable] = value
+            setattr(self.model, slider_spec.variable, value)
             slider.indicator.setText(str(value))
 
         slider.sliderBar.valueChanged.connect(update_variable)
@@ -459,7 +495,7 @@ class Application:
 
         checkbox.stateChanged.connect(update_variable)
         row.addWidget(checkbox)
-        self.controllers.append(slider)
+        self.controllers.append(checkbox)
 
     def add_monitor(self, monitor_spec, plots_box):
         monitor = Monitor(monitor_spec.variable, self.model)
@@ -481,11 +517,6 @@ class Application:
         plots_box.addWidget(histogram)
         self.model.plots.add(histogram)
 
-    def add_render_toggle(self, rowbox):
-        self.toggle_render_btn = QtWidgets.QPushButton("Disable rendering")
-        self.toggle_render_btn.setCheckable(True)
-        rowbox.addWidget(self.toggle_render_btn)
-
     def add_controllers(self, rows, controller_box):
         first_row = True
         for row in rows:
@@ -493,7 +524,6 @@ class Application:
             rowbox = QtWidgets.QHBoxLayout()
             controller_box.addLayout(rowbox)
             if first_row:
-                self.add_render_toggle(rowbox)
                 first_row = False
 
             # Add controllers
@@ -527,10 +557,14 @@ def run(model):
     # We need to store a reference to the application, even though we are not
     # using it, as otherwise it will be garbage-collected and the UI will get
     # some very weird behavior.
-    app = Application(model)
+    app = Application(model)  # noqa: F841
 
     # Launch the application
     qapp.exec_()
 
     # Application was closed, clean up and exit
     sys.exit(0)
+
+
+def quick_run():
+    run(get_quickstart_model())
